@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -102,6 +102,50 @@ class VerticaType(object):
     LONGVARBINARY = 116
     BINARY = 117
 
+    ROW = 300
+    ARRAY = 301 # multidimensional array or contain ROWs
+    MAP = 302
+
+    # one-dimensional array of a primitive type
+    ARRAY1D_BOOL = 1505
+    ARRAY1D_INT8 = 1506
+    ARRAY1D_FLOAT8 = 1507
+    ARRAY1D_CHAR = 1508
+    ARRAY1D_VARCHAR = 1509
+    ARRAY1D_DATE = 1510
+    ARRAY1D_TIME = 1511
+    ARRAY1D_TIMESTAMP = 1512
+    ARRAY1D_TIMESTAMPTZ = 1513
+    ARRAY1D_INTERVAL = 1514
+    ARRAY1D_INTERVALYM = 1521
+    ARRAY1D_TIMETZ = 1515
+    ARRAY1D_NUMERIC = 1516
+    ARRAY1D_VARBINARY = 1517
+    ARRAY1D_UUID = 1520
+    ARRAY1D_BINARY = 1522
+    ARRAY1D_LONGVARCHAR = 1519
+    ARRAY1D_LONGVARBINARY = 1518
+
+    # one-dimensional set of a primitive type
+    SET_BOOL = 2705
+    SET_INT8 = 2706
+    SET_FLOAT8 = 2707
+    SET_CHAR = 2708
+    SET_VARCHAR = 2709
+    SET_DATE = 2710
+    SET_TIME = 2711
+    SET_TIMESTAMP = 2712
+    SET_TIMESTAMPTZ = 2713
+    SET_INTERVAL = 2714
+    SET_INTERVALYM = 2721
+    SET_TIMETZ = 2715
+    SET_NUMERIC = 2716
+    SET_VARBINARY = 2717
+    SET_UUID = 2720
+    SET_BINARY = 2722
+    SET_LONGVARCHAR = 2719
+    SET_LONGVARBINARY = 2718
+
     def __init__(self, *values):
         self.values = values
 
@@ -159,47 +203,116 @@ INTERVAL_MASK_HOUR2MIN = INTERVAL_MASK_HOUR | INTERVAL_MASK_MINUTE
 INTERVAL_MASK_HOUR2SEC = INTERVAL_MASK_HOUR | INTERVAL_MASK_MINUTE | INTERVAL_MASK_SECOND
 INTERVAL_MASK_MIN2SEC = INTERVAL_MASK_MINUTE | INTERVAL_MASK_SECOND
 
+TYPENAME = {
+    VerticaType.UNKNOWN: "Unknown",
+    VerticaType.BOOL: "Boolean",
+    VerticaType.INT8: "Integer",
+    VerticaType.FLOAT8: "Float",
+    VerticaType.CHAR: "Char",
+    VerticaType.VARCHAR: "Varchar",
+    VerticaType.LONGVARCHAR: "Long Varchar",
+    VerticaType.DATE: "Date",
+    VerticaType.TIME: "Time",
+    VerticaType.TIMETZ: "TimeTz",
+    VerticaType.TIMESTAMP: "Timestamp",
+    VerticaType.TIMESTAMPTZ: "TimestampTz",
+    VerticaType.BINARY: "Binary",
+    VerticaType.VARBINARY: "Varbinary",
+    VerticaType.LONGVARBINARY: "Long Varbinary",
+    VerticaType.NUMERIC: "Numeric",
+    VerticaType.UUID: "Uuid",
+    VerticaType.ROW: "Row",
+    VerticaType.ARRAY: "Array",
+    VerticaType.MAP: "Map",
+    VerticaType.ARRAY1D_BOOL: "Array[Boolean]",
+    VerticaType.ARRAY1D_INT8: "Array[Int8]",
+    VerticaType.ARRAY1D_FLOAT8: "Array[Float8]",
+    VerticaType.ARRAY1D_CHAR: "Array[Char]",
+    VerticaType.ARRAY1D_VARCHAR: "Array[Varchar]",
+    VerticaType.ARRAY1D_DATE: "Array[Date]",
+    VerticaType.ARRAY1D_TIME: "Array[Time]",
+    VerticaType.ARRAY1D_TIMESTAMP: "Array[Timestamp]",
+    VerticaType.ARRAY1D_TIMESTAMPTZ: "Array[TimestampTz]",
+    VerticaType.ARRAY1D_TIMETZ: "Array[TimeTz]",
+    VerticaType.ARRAY1D_NUMERIC: "Array[Numeric]",
+    VerticaType.ARRAY1D_VARBINARY: "Array[Varbinary]",
+    VerticaType.ARRAY1D_UUID: "Array[Uuid]",
+    VerticaType.ARRAY1D_BINARY: "Array[Binary]",
+    VerticaType.ARRAY1D_LONGVARCHAR: "Array[Long Varchar]",
+    VerticaType.ARRAY1D_LONGVARBINARY: "Array[Long Varbinary]",
+    VerticaType.SET_BOOL: "Set[Boolean]",
+    VerticaType.SET_INT8: "Set[Int8]",
+    VerticaType.SET_FLOAT8: "Set[Float8]",
+    VerticaType.SET_CHAR: "Set[Char]",
+    VerticaType.SET_VARCHAR: "Set[Varchar]",
+    VerticaType.SET_DATE: "Set[Date]",
+    VerticaType.SET_TIME: "Set[Time]",
+    VerticaType.SET_TIMESTAMP: "Set[Timestamp]",
+    VerticaType.SET_TIMESTAMPTZ: "Set[TimestampTz]",
+    VerticaType.SET_TIMETZ: "Set[TimeTz]",
+    VerticaType.SET_NUMERIC: "Set[Numeric]",
+    VerticaType.SET_VARBINARY: "Set[Varbinary]",
+    VerticaType.SET_UUID: "Set[Uuid]",
+    VerticaType.SET_BINARY: "Set[Binary]",
+    VerticaType.SET_LONGVARCHAR: "Set[Long Varchar]",
+    VerticaType.SET_LONGVARBINARY: "Set[Long Varbinary]",
+}
+
+COMPLEX_ELEMENT_TYPE = {
+    VerticaType.ARRAY1D_BOOL: VerticaType.BOOL,
+    VerticaType.ARRAY1D_INT8: VerticaType.INT8,
+    VerticaType.ARRAY1D_FLOAT8: VerticaType.FLOAT8,
+    VerticaType.ARRAY1D_CHAR: VerticaType.CHAR,
+    VerticaType.ARRAY1D_VARCHAR: VerticaType.VARCHAR,
+    VerticaType.ARRAY1D_DATE: VerticaType.DATE,
+    VerticaType.ARRAY1D_TIME: VerticaType.TIME,
+    VerticaType.ARRAY1D_TIMESTAMP: VerticaType.TIMESTAMP,
+    VerticaType.ARRAY1D_TIMESTAMPTZ: VerticaType.TIMESTAMPTZ,
+    VerticaType.ARRAY1D_TIMETZ: VerticaType.TIMETZ,
+    VerticaType.ARRAY1D_INTERVAL: VerticaType.INTERVAL,
+    VerticaType.ARRAY1D_INTERVALYM: VerticaType.INTERVALYM,
+    VerticaType.ARRAY1D_NUMERIC: VerticaType.NUMERIC,
+    VerticaType.ARRAY1D_VARBINARY: VerticaType.VARBINARY,
+    VerticaType.ARRAY1D_UUID: VerticaType.UUID,
+    VerticaType.ARRAY1D_BINARY: VerticaType.BINARY,
+    VerticaType.ARRAY1D_LONGVARCHAR: VerticaType.LONGVARCHAR,
+    VerticaType.ARRAY1D_LONGVARBINARY: VerticaType.LONGVARBINARY,
+    VerticaType.SET_BOOL: VerticaType.BOOL,
+    VerticaType.SET_INT8: VerticaType.INT8,
+    VerticaType.SET_FLOAT8: VerticaType.FLOAT8,
+    VerticaType.SET_CHAR: VerticaType.CHAR,
+    VerticaType.SET_VARCHAR: VerticaType.VARCHAR,
+    VerticaType.SET_DATE: VerticaType.DATE,
+    VerticaType.SET_TIME: VerticaType.TIME,
+    VerticaType.SET_TIMESTAMP: VerticaType.TIMESTAMP,
+    VerticaType.SET_TIMESTAMPTZ: VerticaType.TIMESTAMPTZ,
+    VerticaType.SET_TIMETZ: VerticaType.TIMETZ,
+    VerticaType.SET_INTERVAL: VerticaType.INTERVAL,
+    VerticaType.SET_INTERVALYM: VerticaType.INTERVALYM,
+    VerticaType.SET_NUMERIC: VerticaType.NUMERIC,
+    VerticaType.SET_VARBINARY: VerticaType.VARBINARY,
+    VerticaType.SET_UUID: VerticaType.UUID,
+    VerticaType.SET_BINARY: VerticaType.BINARY,
+    VerticaType.SET_LONGVARCHAR: VerticaType.LONGVARCHAR,
+    VerticaType.SET_LONGVARBINARY: VerticaType.LONGVARBINARY,
+}
 
 def getTypeName(data_type_oid, type_modifier):
     """Returns the base type name according to data_type_oid and type_modifier"""
-
-    if data_type_oid == VerticaType.BOOL:
-        return "Boolean"
-    elif data_type_oid == VerticaType.INT8:
-        return "Integer"
-    elif data_type_oid == VerticaType.FLOAT8:
-        return "Float"
-    elif data_type_oid == VerticaType.CHAR:
-        return "Char"
-    elif data_type_oid in (VerticaType.VARCHAR, VerticaType.UNKNOWN):
-        return "Varchar"
-    elif data_type_oid == VerticaType.LONGVARCHAR:
-        return "Long Varchar"
-    elif data_type_oid == VerticaType.DATE:
-        return "Date"
-    elif data_type_oid == VerticaType.TIME:
-        return "Time"
-    elif data_type_oid == VerticaType.TIMETZ:
-        return "TimeTz"
-    elif data_type_oid == VerticaType.TIMESTAMP:
-        return "Timestamp"
-    elif data_type_oid == VerticaType.TIMESTAMPTZ:
-        return "TimestampTz"
+    if data_type_oid in TYPENAME:
+        return TYPENAME[data_type_oid]
     elif data_type_oid in (VerticaType.INTERVAL, VerticaType.INTERVALYM):
         return "Interval " + getIntervalRange(data_type_oid, type_modifier)
-    elif data_type_oid == VerticaType.BINARY:
-        return "Binary"
-    elif data_type_oid == VerticaType.VARBINARY:
-        return "Varbinary"
-    elif data_type_oid == VerticaType.LONGVARBINARY:
-        return "Long Varbinary"
-    elif data_type_oid == VerticaType.NUMERIC:
-        return "Numeric"
-    elif data_type_oid == VerticaType.UUID:
-        return "Uuid"
+    elif data_type_oid in (VerticaType.ARRAY1D_INTERVAL, VerticaType.ARRAY1D_INTERVALYM):
+        return "Array[Interval {}]".format(getIntervalRange(COMPLEX_ELEMENT_TYPE[data_type_oid], type_modifier))
+    elif data_type_oid in (VerticaType.SET_INTERVAL, VerticaType.SET_INTERVALYM):
+        return "Set[Interval {}]".format(getIntervalRange(COMPLEX_ELEMENT_TYPE[data_type_oid], type_modifier))
     else:
         return "Unknown"
 
+def getComplexElementType(data_type_oid):
+    """For 1D ARRAY or SET, returns the type of its elements"""
+    return COMPLEX_ELEMENT_TYPE.get(data_type_oid)
 
 def getIntervalRange(data_type_oid, type_modifier):
     """Extracts an interval's range from the bits set in its type_modifier"""
