@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -70,9 +70,19 @@ class Column(object):
         self.null_ok = col['null_ok']
         self.is_identity = col['is_identity']
         self.format_code = col['format_code']
+        self.child_columns = None
         self.props = ColumnTuple(self.name, self.type_code, self.display_size, self.internal_size,
                                  self.precision, self.scale, self.null_ok)
 
+    def add_child_column(self, col):
+        """
+        Complex types involve multiple columns arranged in a hierarchy of parents and children.
+        Each parent column stores references to child columns in a list.
+        """
+        if self.child_columns is None:
+            self.child_columns = []
+        self.child_columns.append(col)
+
     def __str__(self):
         return as_str(str(self.props))
 
