@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -45,7 +45,6 @@ The response is either BindComplete or ErrorResponse.
 from __future__ import print_function, division, absolute_import
 
 from struct import pack
-from six import string_types
 
 from ..message import BulkFrontendMessage
 from ....datatypes import VerticaType
@@ -97,7 +96,7 @@ class Bind(BulkFrontendMessage):
                 # Convert input to string
                 if oid == VerticaType.BOOL:
                     val = '1' if str(val).lower() in ('t', 'true', 'y', 'yes', '1') else '0'
-                elif not isinstance(val, (string_types, bytes)):
+                elif not isinstance(val, (str, bytes)):
                     val = str(val)
                 # Encode string as UTF8 bytes
                 val = val.encode('utf-8') if not isinstance(val, bytes) else val
